@@ -14,6 +14,8 @@
 # along with `dolfin_navier_scipy`.
 # If not, see <http://www.gnu.org/licenses/>.
 
+import logging
+
 import dolfin
 import dolfin_navier_scipy.dolfin_to_sparrays as dts
 import numpy as np
@@ -162,16 +164,17 @@ def get_sysmats(problem='gen_bccont', scheme=None, ppin=None,
 
     # remove the freedom in the pressure if required
     if problem == 'cylinderwake':
-        print('cylinderwake: pressure need not be pinned')
+        logging.debug('cylinderwake: pressure need not be pinned')
         if ppin is not None:
             raise UserWarning('pinning the p will give wrong results')
     elif ppin is None:
-        print('pressure is not pinned - `J` may be singular for internal flow')
+        logging.debug('pressure is not pinned - ' +
+                      '`J` may be singular for internal flow')
     elif ppin == -1:
         stokesmats['J'] = stokesmats['J'][:-1, :][:, :]
         stokesmats['JT'] = stokesmats['JT'][:, :-1][:, :]
         rhsd_vf['fp'] = rhsd_vf['fp'][:-1, :]
-        print('pressure pinned at last dof `-1`')
+        logging.info('pressure pinned at last dof `-1`')
     else:
         raise NotImplementedError('Cannot pin `p` other than at `-1`')
 
@@ -799,7 +802,7 @@ def gen_bccont_fems(scheme='TH', bccontrol=True, verbose=False,
     """
 
     # Load mesh
-    print('mesh: ' + strtomeshfile)
+    logging.info('mesh: ' + strtomeshfile)
     mesh = dolfin.Mesh(strtomeshfile)
 
     if scheme == 'CR':
@@ -819,7 +822,7 @@ def gen_bccont_fems(scheme='TH', bccontrol=True, verbose=False,
     inflowgeodata = cntbcsdata['inflow']
 
     inflwpe = inflowgeodata['physical entity']
-    print('mesh: physical entity {0} -- inflow'.format(inflwpe))
+    logging.info('mesh: physical entity {0} -- inflow'.format(inflwpe))
     inflwin = np.array(inflowgeodata['inward normal'])
     inflwxi = np.array(inflowgeodata['xone'])
     inflwxii = np.array(inflowgeodata['xtwo'])
@@ -843,15 +846,15 @@ def gen_bccont_fems(scheme='TH', bccontrol=True, verbose=False,
     for wpe in wallspel:
         diribcu.append(dolfin.DirichletBC(V, gzero, boundaries, wpe))
         bcdict = diribcu[-1].get_boundary_values()
-        print('mesh: physical entity {0} -- wall'.format(wpe))
+        logging.info('mesh: physical entity {0} -- wall'.format(wpe))
 
     if not bccontrol:  # treat the control boundaries as walls
         try:
             for cntbc in cntbcsdata['controlbcs']:
                 diribcu.append(dolfin.DirichletBC(V, gzero, boundaries,
                                                   cntbc['physical entity']))
-                print('mesh: physical entity {0} -- wall'.
-                      format(cntbc['physical entity']))
+                logging.info('mesh: physical entity {0} -- wall'.
+                             format(cntbc['physical entity']))
         except KeyError:
             pass  # no control boundaries
 
@@ -869,19 +872,19 @@ def gen_bccont_fems(scheme='TH', bccontrol=True, verbose=False,
                 raise NotImplementedError()
             mvwdbcs.append(dolfin.DirichletBC(V, rotcyl, boundaries,
                                               cntbc['physical entity']))
-            print('mesh: physical entity {0} -- moving wall'.
-                  format(cntbc['physical entity']))
+            logging.info('mesh: physical entity {0} -- moving wall'.
+                         format(cntbc['physical entity']))
     except KeyError:
         pass  # no moving walls defined
     if not movingwallcntrl and len(mvwdbcs) > 0:
         diribcu.extend(mvwdbcs)  # add the moving walls to the diri bcs
         mvwdbcs = []
-        print('mesh: physical entities -- moving walls --> walls')
+        logging.info('mesh: physical entities -- moving walls --> walls')
 
     # Create outflow boundary condition for pressure
     # TODO XXX why zero pressure?? is this do-nothing???
     outflwpe = cntbcsdata['outflow']['physical entity']
-    print('mesh: physical entity {0} -- outflow'.format(outflwpe))
+    logging.info('mesh: physical entity {0} -- outflow'.format(outflwpe))
     g2 = dolfin.Constant(0)
     bc2 = dolfin.DirichletBC(Q, g2, boundaries, outflwpe)
 
@@ -921,8 +924,8 @@ def gen_bccont_fems(scheme='TH', bccontrol=True, verbose=False,
                 csf = RotatingCircle(center=cbc['center'],
                                      radius=cbc['radius'])
             cpe = cbc['physical entity']
-            print('mesh: physical entity {0} -- boundary control ({1})'.
-                  format(cpe, cbc['type']))
+            logging.info('mesh: physical entity {0} -- boundary control ({1})'.
+                         format(cpe, cbc['type']))
             bcshapefuns.append(csf)
             bcpes.append(cpe)
             bcds.append(dolfin.Measure("ds", subdomain_data=boundaries)(cpe))
@@ -932,7 +935,7 @@ def gen_bccont_fems(scheme='TH', bccontrol=True, verbose=False,
         ldsurfpe = cntbcsdata['lift drag surface']['physical entity']
         liftdragds = dolfin.Measure("ds", subdomain_data=boundaries)(ldsurfpe)
         bclds = dolfin.DirichletBC(V, gzero, boundaries, ldsurfpe)
-        print('mesh: physical entity {0} -- lift/drag surf'.format(ldsurfpe))
+        logging.info(f'mesh: physical entity {ldsurfpe} -- lift/drag surf')
         bcldsdict = bclds.get_boundary_values()
         ldsbcinds = list(bcldsdict.keys())
     except KeyError:
@@ -1093,13 +1096,12 @@ class RotatingCircle(dolfin.UserExpression):
 
 class LiftDragSurfForce():
 
-    def __init__(self, V=None, nu=None, ldds=None,
+    def __init__(self, V=None, nu=None, ldds=None, gradvsymmtrc=True,
                  outflowds=None, phione=None, phitwo=None):
         self.mesh = V.mesh()
         self.n = dolfin.FacetNormal(self.mesh)
         # self.Id = dolfin.Identity(self.mesh.geometry().dim())
-        self.ldds = ldds
-        self.outflowds = outflowds
+        # self.ldds = ldds
         self.nu = nu
         self.A = dolfin.as_matrix([[0., 1.],
                                    [-1., 0.]])
@@ -1108,10 +1110,15 @@ class LiftDragSurfForce():
         self.phionex = pickx*phione
         self.phioney = picky*phione
         self.phitwo = phitwo
+        self.gradvsymmtrc = gradvsymmtrc
 
-        def epsilon(u):
-            return 0.5*(dolfin.nabla_grad(u) + dolfin.nabla_grad(u).T)
-
+        if gradvsymmtrc:
+            def epsilon(u):
+                return 0.5*(dolfin.nabla_grad(u) + dolfin.nabla_grad(u).T)
+            self.outflowds = outflowds
+        else:
+            def epsilon(u):
+                return dolfin.grad(u)
         self.epsilon = epsilon
 
     def evaliftdragforce(self, u=None, p=None):
@@ -1137,24 +1144,30 @@ class LiftDragSurfForce():
 
         # ## the Babuska/Millner trick
         pox = self.phionex
+        poy = self.phioney
 
         # convection
         drgo = inner(dolfin.dot(u, dolfin.nabla_grad(u)), pox)*dx
-        # outflow correction
-        ofcor = (self.nu*inner(grad(u).T*self.n, pox))*self.outflowds
+        # outflow correction for symmetrized gradient
+        if self.gradvsymmtrc:
+            dofcor = (self.nu*inner(grad(u).T*self.n, pox))*self.outflowds
+            lofcor = (self.nu*inner(grad(u).T*self.n, poy))*self.outflowds
+        else:
+            dofcor, lofcor = 0, 0
+
         # diffusion
         drgt = (2*self.nu*inner(epsi(u), grad(self.phionex)))*dx
         # pressure
         drgd = -p*div(self.phionex)*dx
 
-        drag = dolfin.assemble(drgt+drgd+drgo-ofcor)
+        drag = dolfin.assemble(drgt+drgd+drgo-dofcor)
+
+        lfto = inner(dolfin.dot(u, dolfin.nabla_grad(u)), poy)*dx
+        lftt = 2*self.nu*inner(self.epsilon(u), dolfin.grad(poy))*dx
+        lftd = (-p*dolfin.div(poy))*dx
+
+        lift = dolfin.assemble(lfto+lftt+lftd-lofcor)
 
-        lfto = inner(dolfin.dot(u, dolfin.nabla_grad(u)), self.phioney)*dx
-        lftt = 2*self.nu*inner(self.epsilon(u), dolfin.grad(self.phioney))*dx
-        lftd = (-p*dolfin.div(self.phioney))*dx
-        # outflow correction
-        ofcor = (self.nu*inner(grad(u).T*self.n, self.phioney))*self.outflowds
-        lift = dolfin.assemble(lfto+lftt+lftd-ofcor)
         return lift, drag
 
     def evatorqueSphere2D(self, u=None, p=None):
@@ -1392,3 +1405,15 @@ def gen_bccont_fems_3D(scheme='TH', bccontrol=True, verbose=False,
                    mesh=mesh)
 
     return gbcfems
+
+
+def get_bcinds(mesh=None, V=None, prfile=None, pelist=[]):
+    boundaries = dolfin.MeshFunction('size_t', mesh, prfile)
+    gzero = dolfin.Constant((0, 0))
+    diribcinds = []
+    for wpe in pelist:
+        cdbc = dolfin.DirichletBC(V, gzero, boundaries, wpe)
+        bcdict = cdbc.get_boundary_values()
+        diribcinds.extend(list(bcdict.keys()))
+
+    return diribcinds
