@@ -1,3 +1,4 @@
+import logging
 import numpy as np
 import scipy.sparse as sps
 import os
@@ -559,6 +560,7 @@ def solve_nse(A=None, M=None, J=None, JT=None,
               feedbackthroughdict=None,
               return_vp=False,
               b_mat=None, cv_mat=None,
+              vp_output=False, vp_out_fun=None, vp_output_dict=None,
               vel_nwtn_stps=20, vel_nwtn_tol=5e-15,
               nsects=1, loc_nwtn_tol=5e-15, loc_pcrd_stps=True,
               addfullsweep=False,
@@ -575,6 +577,8 @@ def solve_nse(A=None, M=None, J=None, JT=None,
               dictkeysstr=False,
               treat_nonl_explicit=True, no_data_caching=True,
               treat_nonl_explct=False,  # TODO: remove deprecated option
+              use_custom_nonlinearity=False,
+              custom_nonlinear_vel_function=None,
               datatrange=None, dataoutpnts=None,
               return_final_vp=False,
               return_as_list=False, return_vp_dict=False,
@@ -660,6 +664,12 @@ def solve_nse(A=None, M=None, J=None, JT=None,
         as initial value, defaults to `False`
     treat_nonl_explicit= string, optional
         whether to treat the nonlinearity explicitly, defaults to `False`
+    use_custom_nonlinearity: boolean, optional
+        whether to use a custom nonlinear velocity function to replace the
+        convection part, defaults to `False`
+    custom_nonlinear_vel_function: callable f(v), optional
+        the custom nonlinear function (cp. `use_custom_nonlinearity`),
+        defaults to `None`
     nsects: int, optional
         in how many segments the trange is split up. (The newton iteration
         will be confined to the segments and, probably, converge faster than
@@ -1013,14 +1023,27 @@ def solve_nse(A=None, M=None, J=None, JT=None,
         def rhsp(t):
             return fp
 
-        def nonlvfunc(vvec):
-            _, convvec, _ = \
-                get_v_conv_conts(vvec=vvec, V=V,
-                                 invinds=dbcntinvinds, semi_explicit=True)
-            return convvec
+        if use_custom_nonlinearity:
+            nonlvfunc = custom_nonlinear_vel_function
+            logging.\
+                debug('The convection is replaced by a custom nonlinearity')
+        else:
+            def nonlvfunc(vvec):
+                _, convvec, _ = \
+                    get_v_conv_conts(vvec=vvec, V=V,
+                                     invinds=dbcntinvinds, semi_explicit=True)
+                return convvec
 
         f_vdp = None if stokes_flow else nonlvfunc
 
+        def _addoutput(vvec, pvec, time=None):
+            if vp_output:
+                vpo = vp_out_fun(vvec, pvec, time=None)
+                vp_output_dict.update({time: vpo})
+            else:
+                pass
+            return
+
         def getbcs(time, vvec, pvec, mode=None):
             return _comp_cntrl_bcvals(time=time, vel=vvec, p=pvec,
                                       diricontbcvals=diricontbcvals,
@@ -1036,16 +1059,24 @@ def solve_nse(A=None, M=None, J=None, JT=None,
             vp_dict = {}
 
             def _svpplz(vvec, pvec, time=None):
+                _addoutput(vvec, pvec, time=time)
                 vp_dict.update({time: dict(p=pvec, v=vvec)})
                 prvoutdict.update(dict(vc=vvec, pc=pvec, t=time))
                 dou.output_paraview(**prvoutdict)
 
         elif return_dictofvelstrs:
             def _svpplz(vvec, pvec, time=None):
+                _addoutput(vvec, pvec, time=time)
                 cfvstr = data_prfx + '_prs_t{0}'.format(time)
                 cfpstr = data_prfx + '_vel_t{0}'.format(time)
-                dou.save_npa(pvec, fstring=cfpstr)
-                dou.save_npa(vvec, fstring=cfvstr)
+                try:
+                    if dataoutpnts is not None and not time == datatrange[0]:
+                        pass
+                    else:
+                        dou.save_npa(pvec, fstring=cfpstr)
+                        dou.save_npa(vvec, fstring=cfvstr)
+                except IndexError:
+                    pass  # if something goes wrong, don't stop
                 _atdct(expnlveldct, time, cfvstr)
                 prvoutdict.update(dict(vc=vvec, pc=pvec, t=time))
                 dou.output_paraview(**prvoutdict)
@@ -1054,6 +1085,7 @@ def solve_nse(A=None, M=None, J=None, JT=None,
             ylist = []
 
             def _svpplz(vvec, pvec, time=None):
+                _addoutput(vvec, pvec, time=time)
                 prvoutdict.update(dict(vc=vvec, pc=pvec, t=time))
                 dou.output_paraview(**prvoutdict)
                 if return_y_list:
@@ -1066,8 +1098,7 @@ def solve_nse(A=None, M=None, J=None, JT=None,
             timintsc = tiu.cnab
         elif time_int_scheme == 'sbdf2':
             timintsc = tiu.sbdftwo
-        if verbose:
-            print('INFO: time integration with', time_int_scheme)
+        logging.info('Time integration with ' + time_int_scheme)
 
         if closed_loop:
             if dynamic_feedback:
@@ -1108,6 +1139,8 @@ def solve_nse(A=None, M=None, J=None, JT=None,
 
             elif static_feedback:
                 pass
+        else:
+            pass
 
         if not dyn_fb_disc == 'linear_implicit':
             icd = dict(f_tdp=rhsv, inivel=iniv, verbose=verbose,
